# -*- encoding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import User

from webadmin.backup.models import Backup
from webadmin.tools import SysBackup, Task

def update_context():
    '''
    Update context dictionary with
    common informations used to renderize
    HTML templates.
    '''
    try:
        bkp_all = Backup.objects.all()
        total = bkp_all.count()
        context = {'total': total}
        if total <=1:
            context.update({'modulo': 'Backup',
                            'extra': 'criado',
                            'backups': bkp_all,
                            })
        else:
            context.update({'modulo': 'Backups',
                            'extra': 'criados',
                            'backups': bkp_all,
                            })
    except Backup.DoesNotExist:
        context = {}
    
    return context
            

@login_required
@permission_required('backup.can_view')
def list(request):
    context = {}

    if request.method == 'POST':
        context = new_backup(request)
#        context = {'msg': u'Backup criado com sucesso.'}

    context.update(update_context())
    return render_to_response('backup/backup_list.html', context,
                              context_instance=RequestContext(request))

@login_required
@permission_required('backup.can_view')
def new_backup(request):
    '''
    Create a new backup file with
    settings defined in tools.py
    '''
    context = {}

    sys_bkp = SysBackup()
    backup = sys_bkp.new_backup()

    if backup == 'Permission denied':
        errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
        context.update({'errmsg': errmsg})
        return context

    db_bkp = Backup()
    db_bkp.name = backup['name']
    db_bkp.date = backup['date']
    db_bkp.size = backup['size']
    db_bkp.save()

    context = {'msg': u'Backup criado com sucesso.'}
    context.update(update_context())
    return context

@login_required
@permission_required('backup.can_view')
def del_backup(request, backup_id):
    '''
    Remove backup file from disk and database
    '''
    context = {}

    try:
        bkp = Backup.objects.get(id=backup_id)

        task = Task()
        backup = task.removeBackupFile(bkp.name)

        if backup == 'Permission denied':
            context = {'errmsg': u'Sem permissão de escrita no arquivo de tarefas!'}
            context.update(update_context())
            return render_to_response('backup/backup_list.html', context,
                                      context_instance=RequestContext(request))

        bkp.delete()
        context = {'msg': u'Backup removido com sucesso.'}
        context.update(update_context())
        return render_to_response('backup/backup_list.html', context,
                                  context_instance=RequestContext(request))
    except Backup.DoesNotExist:
        pass

    if request.method == 'POST':
        context = new_backup(request)

    context.update(update_context())
    return render_to_response('backup/backup_list.html', context,
                              context_instance=RequestContext(request))

@login_required
@permission_required('backup.can_view')
def restore_backup(request, backup_id):
    '''
    Restore backup file
    '''
    
    try:
        bkp = Backup.objects.get(id=backup_id)

        task = Task()
        backup = task.restoreBackupFile(bkp.name)

        context = {'msg': u'Backup %s restaurado com sucesso.' % bkp.name.split('/')[2]}
        context.update(update_context())
        return render_to_response('backup/backup_list.html', context,
                                  context_instance=RequestContext(request))
    except Backup.DoesNotExist:
        context = update_context()
        return render_to_response('backup/backup_list.html', context,
                                  context_instance=RequestContext(request))
