# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required

from webadmin.firewall.models import Firewall, BlockApp
from webadmin.firewall.forms import FormFirewall, FormBlockApp
from webadmin.tools import System, Task

def update_context():
    """
    Update the context dictonary with
    commom informations to be rendered in the HTML template
    """
    context = {'modulo': 'Bloqueios',
               'extra': 'Telecentros.BR',
               }
    try:
        blocked_urls = Firewall.objects.all()
        context.update({'urls': blocked_urls})
    except Firewall.DoesNotExist:
        pass

    try:
        blocked_apps = BlockApp.objects.all()
        context.update({'apps': blocked_apps})
    except BlockApp.DoesNotExist:
        pass

    return context

@login_required
def block_apps(request):
    if request.method == 'POST':
        form = FormBlockApp(request.POST, request.FILES)
        context ={'form': form}
        context.update(update_context())
        if form.is_valid():
            rules = form.save(commit=False)
            app = form.cleaned_data['app']

            task = Task()
            ret = task.blockApp(app, True)
            if ret != 1:
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg})
                return render_to_response('system/security/apps.html', context,
                                          context_instance=RequestContext(request))
            else:
                rules.save()
                mensagem = u'Aplicativo %s adicionado a lista de bloqueios.' % app
                context.update({'msg': mensagem})
                return render_to_response('system/security/apps.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = FormBlockApp()
        context = update_context()
        context.update({'modulo': 'Bloqueios',
                        'extra': 'Aplicativos',
                        'form': form})
        return render_to_response('system/security/apps.html', context,
                                  context_instance=RequestContext(request))

@login_required
def block_sites(request):
    if request.method == 'POST':
        form = FormFirewall(request.POST, request.FILES)
        context ={'form': form}
        context.update(update_context())
        if form.is_valid():
            rules = form.save(commit=False)
            url = form.cleaned_data['url']

            task = Task()
            ret = task.blockUrl(url, True)
            if ret != 1:
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg})
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
            else:
                rules.save()
                mensagem = u'Página %s adicionada a lista de bloqueios.' % url
                context.update({'msg': mensagem})
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = FormFirewall()
        context = update_context()
        context.update({'modulo': 'Bloqueios',
                        'extra': 'Sites',
                        'form': form})
        return render_to_response('system/security/firewall.html', context,
                                  context_instance=RequestContext(request))

@login_required
def alter_app(request, id_rule):
    app = get_object_or_404(BlockApp, pk=id_rule)
    form = FormBlockApp({'app': app})

    if request.method == 'POST':
        form = FormBlockApp(request.POST, instance=app)
        if form.is_valid():
            app = form.save(commit=False)
            new_app_name = form.cleaned_data['app']
            app.url = new_app_name
            task = Task()
            ret = task.blockApp(app.app, True)
            if ret != 1:
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg})
                return render_to_response('system/security/apps.html', context,
                                          context_instance=RequestContext(request))
            else:
                form.save()
                message = u'Aplicativo %s alterado com sucesso.' % new_app_name
                context = {'msg': message,
                           'form': form}
                context.update(update_context())
                return render_to_response('system/security/apps.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = FormBlockApp(instance=app)
        context = {'form': form}

    context.update(update_context())
    return render_to_response('system/security/apps.html', context,
                              context_instance=RequestContext(request))

@login_required
def remove_app(request, id_rule):
    context = update_context()
    try:
        app = BlockApp.objects.get(id=id_rule)
        task = Task()
        ret = task.blockApp(app.app, False)
        if ret != 1:
            errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
            context.update({'errmsg': errmsg})
            return render_to_response('system/security/apps.html', context,
                                      context_instance=RequestContext(request))
        else:
            app.delete()
            form = FormBlockApp()
            message = u'Aplicativo %s removido da lista de bloqueios.' % app.app
            context = {'msg': message,
                       'form': form}
            context.update(update_context())
            return render_to_response('system/security/apps.html', context,
                                      context_instance=RequestContext(request))
    except BlockApp.DoesNotExist:
        pass
    
    return render_to_response('system/security/apps.html', {},
                              context_instance=RequestContext(request))

@login_required
def alter_site(request, id_rule):
    site = get_object_or_404(Firewall, pk=id_rule)
    form = FormFirewall({'url': site})

    if request.method == 'POST':
        form = FormFirewall(request.POST, instance=site)
        if form.is_valid():
            site = form.save(commit=False)
            new_site_name = form.cleaned_data['url']
            site.url = new_site_name
            task = Task()
            ret = task.blockUrl(site.url, True)
            if ret != 1:
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg})
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
            else:
                form.save()
                message = u'Página %s alterado com sucesso.' % new_site_name
                context = {'msg': message,
                       'form': form}
                context.update(update_context())
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = FormFirewall(instance=site)
        context = {'form': form}

    context.update(update_context())
    return render_to_response('system/security/firewall.html', context,
                              context_instance=RequestContext(request))

@login_required
def remove_site(request, id_rule):
    context = update_context()
    try:
        site = Firewall.objects.get(id=id_rule)
        task = Task()
        ret = task.blockUrl(site.url, False)
        if ret != 1:
            errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
            context.update({'errmsg': errmsg})
            return render_to_response('system/security/firewall.html', context,
                                      context_instance=RequestContext(request))
        else:
            site.delete()
            form = FormFirewall()
            message = u'Página %s removida da lista de bloqueios.' % site.url
            context = {'msg': message,
                       'form': form}
            context.update(update_context())
            return render_to_response('system/security/firewall.html', context,
                                      context_instance=RequestContext(request))
    except Firewall.DoesNotExist:
        pass
    
    return render_to_response('system/security/firewall.html', {},
                              context_instance=RequestContext(request))
