# -*- encoding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required

from django.contrib.auth.models import User
from models import Group
from forms import FormGrupo

from webadmin.tools import Task

def update_context():
    """
    Atualiza o dicionario com informações
    comuns que serão renderizados nos templates HTML
    """
    try:
        """
        Lista todos os grupos
        """
        g_all = Group.objects.all()
        usuarios = [g.usuario.all() for g in g_all]
        total = g_all.count()
        context = {'total': total}
        if total == 1:
            context.update({'modulo': 'Grupo',
                            'extra': 'cadastrado',
                            'grupos': g_all,
                            'usuarios': usuarios,
                            })
        else:
            context.update({'modulo': 'Grupos',
                            'extra': 'cadastrados',
                            'grupos': g_all,
                            'usuarios': usuarios,
                            })

    except User.DoesNotExist:
        context = {}

    return context

@login_required
def listing(request):
    context = {}
    context.update(update_context())
    return render_to_response('group/list.html', context,
                              context_instance=RequestContext(request))

@login_required
def include(request):
    if request.method == 'POST':
        form = FormGrupo(request.POST, request.FILES)
        context = {'form': form}

        if form.is_valid():
            group_name = form.cleaned_data['nome']

            task = Task()
            status = task.addGroup(group_name)
            if status == 'Permission denied':
                context.update({
                        'errmsg': u'Sem permissão de escrita no arquivo de tarefas!'})
                context.update(update_context())
                return render_to_response('group/include.html', context,
                                          context_instance=RequestContext(request))

            form.save()

            users_objs = Group.objects.get(nome=group_name).usuario.all()
            users = [ u.username for u in users_objs ]	# User list for current group
            for user in users:
                task.addToGroup(user, group_name)

            mensagem = u'Grupo %s criado com sucesso.' % group_name
            context = {'msg': mensagem, 
                       'form': FormGrupo()}
            context.update(update_context())
            return render_to_response('group/include.html', context,
                                      context_instance=RequestContext(request))
    else:
        form = FormGrupo()
        context = {'form': form}

    context.update(update_context())
    return render_to_response('group/include.html', context,
                              context_instance=RequestContext(request))

@login_required
def alter(request, id_grupo):
    """
    Altera o grupo selecionado com id_grupo
    """
    grupo_nome = get_object_or_404(Group, pk=id_grupo)
    oldname = grupo_nome.nome
    form = FormGrupo({'nome': grupo_nome})
    context = {'form': form}
    users_objs = Group.objects.get(nome=grupo_nome).usuario.all()
    before_users = [ u.username for u in users_objs ]

    if request.method == 'POST':
        form = FormGrupo(request.POST, instance=grupo_nome)
        if form.is_valid():
            grupo = form.save(commit=False)
            novo_nome_grupo = form.cleaned_data['nome']
            usuarios = form.cleaned_data['usuario']

            task = Task()
            # If group has new name...
            if oldname != novo_nome_grupo:
                status = task.renameGroup(oldname, novo_nome_grupo)
                if status == 'Permission denied':
                    context.update({
                            'errmsg': u'Sem permissão de escrita no arquivo de tarefas!'})
                    context.update(update_context())
                    return render_to_response('group/include.html', context,
                                              context_instance=RequestContext(request))

            grupo.nome = novo_nome_grupo
            [ grupo.usuario.add(u.id) for u in usuarios ]

            form.save()
            form.save_m2m()

                # PRECISA CHECAR SE A LISTA DE USUARIOS MUDOU
                # SE SIM, FAZER REMOCAO OU INSERCAO DE USUARIOS
                # NO GRUPO. SENAO TERMINA            
            users_objs = Group.objects.get(nome=novo_nome_grupo).usuario.all()
            after_users = [ u.username for u in users_objs ]
            if before_users != after_users:
                for user in after_users:
                    # o ideal seria pegar somente os itens novos e excluidos
                    # para evitar sobrecarga do LDAP
                    task.addToGroup(user, novo_nome_grupo)

            mensagem = u'Grupo %s alterado com sucesso.' % novo_nome_grupo
            context = {'msg': mensagem}
            context.update(update_context())
            return render_to_response('group/list.html', context,
                                      context_instance=RequestContext(request))
    else:
        form = FormGrupo(instance=grupo_nome)
        context = {'form': form}

    context.update(update_context())
    return render_to_response('group/alter.html', context,
                              context_instance=RequestContext(request))

@login_required
def remove(request, id_grupo):
    """
    Remove o grupo com id = id_grupo.
    """
    try:
        g = Group.objects.get(id=id_grupo)
        task = Task()
        status = task.removeGroup(g)
        if status == 'Permission denied':
            context = {'errmsg': u'Sem permissão de escrita no arquivo de tarefas!'}
            context.update(update_context())
            return render_to_response('group/list.html', context,
                                      context_instance=RequestContext(request))

        g.delete()
        mensagem = u'Grupo %s removido.' % g.nome
        context = {'msg': mensagem}
        context.update(update_context())
        return render_to_response('group/list.html', context,
                                  context_instance=RequestContext(request))
    except Group.DoesNotExist:
        pass
    
    return render_to_response('group/list.html', {},
                              context_instance=RequestContext(request))
