# -*- encoding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import User

from webadmin.system.models import Address
from webadmin.system.forms import FormRegistration, FormAdmPasswd
from webadmin.tools import System, Task

def update_context():
    """
    Update the context dictonary with
    commom informations to be rendered in the HTML template
    """
    context = {'modulo': 'Telecentro',
               'extra': 'Telecentros.BR',
               }

    try:
        telecentro = Address.objects.get()
        context.update({'telecentro': telecentro})
    except Address.DoesNotExist:
        pass

    return context

def clients(request):
    context = {}
    s = System()
    clients = s.clients()

    if clients == 0:
        context.update({'modulo': 'Gerenciamento',
                        'extra': 'Clientes',
                        'clients': [],
                        'errmsg': 'Erro na configuração da placa de rede interna, ou placa inexistente.',
                        })
    else:
        context.update({'modulo': 'Gerenciamento',
                        'extra': 'Clientes',
                        'clients': clients,
                        })

    return render_to_response('system/clients.html', context,
                              context_instance=RequestContext(request))

@login_required
def printer(request):
    s = System()
    p = s.printer()

    context = update_context()
    context.update(
        {'modulo': 'Gerenciamento',
         'extra': 'Impressoras',
         'printers': p,
         })
    return render_to_response('system/hardware.html', context,
                              context_instance=RequestContext(request))

@login_required
def admin_password(request):

    context = update_context()
    admin_username = request.user
    try:
        admin_obj = User.objects.get(username__exact=admin_username)
    except User.DoesNotExist:
        context.update({'errmsg': 'Erro: Usuário "%s" não existe.' % admin_username})
        return render_to_response('system/adminpasswd.html', context,
                                  context_instance=RequestContext(request))

    if request.method == 'POST':
        form = FormAdmPasswd(request.POST)
        context ={'form': form}
        context.update(update_context())
        if form.is_valid():
            passwd = form.cleaned_data['passwd']

            # Create a task to be managed by a daemon, 
            # to change admin password.
            task = Task()
            ret = task.changePassword(admin_username, passwd)	# return 1 or IOError
            if ret != 1:	# If IOError(permission, file not exist, etc...)
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg, 
                                'adm': admin_obj})
                return render_to_response('system/adminpasswd.html', context,
                                          context_instance=RequestContext(request))
            else:
                admin_obj.set_password(passwd);
                admin_obj.save()
                context.update({'modulo': 'Segurança',
                                'extra': 'senha admin.',
                                'msg': 'Senha alterada com sucesso. Ela terá efeito em sua próxima sessão.',
                                'adm': admin_obj,})
                return render_to_response('system/adminpasswd.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = FormAdmPasswd()

    context.update(
        {'modulo': 'Segurança',
         'extra': 'senha admin.',
         'adm': admin_obj,
         'form': form}
        )
    return render_to_response('system/adminpasswd.html', context,
                              context_instance=RequestContext(request))

@login_required
def security(request):
    context = update_context()
    context.update(
        {'modulo': 'Bloqueios'}
        )
    return render_to_response('system/security.html', context,
                              context_instance=RequestContext(request))

@login_required
def services(request):
    context = update_context()
    context.update(
        {'modulo': 'Serviços'}
        )
    return render_to_response('system/services.html', context,
                              context_instance=RequestContext(request))

@login_required
@permission_required('system.can_view')
def telecentro(request):
    """
    Registers the "telecentro" with its informations.
    """
    registered = Address.objects.all()
    
    def telecentro_register(request):
        if request.method == 'POST':
            form_register = FormRegistration(request.POST, request.FILES)
            context = {'form_register': form_register}
        
            if form_register.is_valid():
                register = form_register.save(commit=False)

                name = form_register.cleaned_data['tl_name']
                register.save()
                mensagem = u'Telecentro %s registrado.' % name

                s = System()
                s.tl_register(name)

                context = {'msg': mensagem,
 #                          'form': FormRegistration(),
                           }

                context.update(update_context())
                id_telecentro = Address.objects.get().id
                return telecentro_display(request, id_telecentro, context)
#                return render_to_response('system/telecentro.html', context,
#                                          context_instance=RequestContext(request))
        else:
            form_register = FormRegistration()
            context = {'form_register': form_register}

        context.update(update_context())
        return render_to_response('system/telecentro_register.html', context,
                                  context_instance=RequestContext(request))
    ## END telecentro_register() ##

    def telecentro_display(request, id_telecentro, context={}):
        telecentro = get_object_or_404(Address, pk=id_telecentro)
        form = FormRegistration(instance=telecentro)
        context.update({'form': form})

        if request.method == 'POST':
            form = FormRegistration(request.POST, instance=telecentro)
            if form.is_valid():
                telecentro = form.save(commit=False)
                telecentro.save()
                mensagem = u'Telecentro %s alterado com sucesso.' % form.cleaned_data['tl_name']
                context.update({'msg': mensagem, 'form': form})
                context.update(update_context())

#        for key in form.fields.iterkeys():
#            form.fields[key].widget.attrs['disabled'] = True
#            form.fields[key].widget.attrs['class'] = 'disabled'
                return render_to_response('system/telecentro.html', context,
                                          context_instance=RequestContext(request))
        else:
            form = FormRegistration(instance=telecentro)
            context.update({'form': form})

        context.update(update_context())
        return render_to_response('system/telecentro.html', context,
                                  context_instance=RequestContext(request))
    ## END telecentro_display() ##


    try:
        id_telecentro = Address.objects.get().id
        return telecentro_display(request, id_telecentro)
    except Address.DoesNotExist:
        return telecentro_register(request)

@login_required
def webcam(request):
    context = update_context()
    ipcam = '192.168.10.3'

    import commands
    status = commands.getstatusoutput('ping -c3 %s' % ipcam)[0]
    if status:
        message = 'Camêra não conectada ou inexistente no endereço %s' % ipcam
        context.update({'errmsg': message})

    context.update(
        {'modulo': 'Gerenciamento',
         'extra': 'Webcam',
         'ip': ipcam,
         })
    return render_to_response('system/webcam.html', context,
                              context_instance=RequestContext(request))
