# -*- encoding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

import sys, os, errno, commands, cups

class System:
    
    def tl_register(self, name):
        '''
        Routine to register telecentro
        '''
        print >> sys.stderr, name
        return name

    def printer(self):
        connection = cups.Connection()
        printers = connection.getPrinters()
        for printer in printers:
            ret = printers[printer]
        return printers

    def clients(self):
        interface = 'eth1'
        raw = commands.getoutput('/usr/sbin/arp -ni %s | sed 1d' % interface)
        raw_list = raw.split('\n')
        clist = []
        labels = ['ip', 'type', 'mac', 'flags', 'iface']

        def inRange(list):
            '''
            Return only the addresses in the range
            192.168.10.10 and 192.168.10.200
            Addresses out of this range are devices
            like printer, router, ipcam.
            '''
            ip = list[0]
            final = ip.split('.').pop()
            return (int(final) > 10 and int(final) < 200)

        if raw:
            for i in raw_list:
                aux = []
                clients = {}
                l = i.split(' ')
                for c in l:
                    if c:
                        aux.append(c)
                if inRange(aux):
                    clients.update(zip(labels, aux))
                    clist.append(clients)
            return clist
        return 0
        
class Task:

    taskfile = '/daemon-files/task.mc'

    def write_to_file(self, values):
        try:
            fd = open(self.taskfile, 'w')
            for data in values:
                fd.write('%s\n' % data)
            fd.close()
        except IOError, e:
            return os.strerror(e.errno)
        
        return 1

    def addUser(self, name, passwd, quota=0):
        # @adduser
        # name
        # group
        # passwd
        # quota
        # squota

        group = 'users'
        try:
            fd = open(self.taskfile, 'w')
            for data in ('@adduser', name, group, passwd, quota, quota):
                fd.write('%s\n' % data)
            fd.close()
        except IOError, e:
            return os.strerror(e.errno)

        return 0

    def alterUser(self, name, passwd, quota=0):
        group = 'users'
        data = ('@alteruser', name, group, passwd, quota, quota)
        return self.write_to_file(data)

    def removeUser(self, name):
        data = ('@removeuser', name)
        return self.write_to_file(data)

    def addGroup(self, group):
        data = ('@addgroup', group)
        return self.write_to_file(data)
 
    def removeGroup(self, group):
        data = ('@removegroup', group)
        return self.write_to_file(data)

    def renameGroup(self, old_name, new_name):
        data = ('@renamegroup', old_name, new_name)
        return self.write_to_file(data)
        
    def addToGroup(self, name, group):
        data = ('@addtogroup', name, group)
        return self.write_to_file(data)
    
    def changePassword(self, username, newpasswd):
        try:
            fd = open(self.taskfile, 'w')
            for data in ('@admpasswd', username, newpasswd):
                fd.write('%s\n' % data)
            fd.close()
        except IOError, e:
            return os.strerror(e.errno)
        
        return 1

    def blockApp(self, app, block):
        if block:
            data = ('@blockapp', 'block', app)
        else:
            data = ('@blockapp', 'unblock', app)
        return self.write_to_file(data)

    def blockUrl(self, url, block):
        if block:
            data = ('@blockurl', 'block', url)
        else:
            data = ('@blockurl', 'unblock', url)
        return self.write_to_file(data)

    def moveBackupFile(self, tarfile):	# Not in use yet.
        destination = '/home/mc'
        data = ('@movefile', tarfile, destination)
        return self.write_to_file(data)
    
    def removeBackupFile(self, tarfile):
        data = ('@removefile', tarfile)
        return self.write_to_file(data)

    def restoreBackupFile(self, tarfile):
        destination = '/'
        data = ('@restorebackup', tarfile, destination)
        return self.write_to_file(data)

class SysBackup:
#    filelist = ['/home/professor/.config']
    filelist = ['/etc/apache2']	# Files to be in backup

    def GetHumanReadable(self, size, precision=2):
        # Return a humanized string representation of a number of bytes.
        suffixes = ['B', 'KB', 'MB', 'GB', 'TB']
        suffixIndex = 0
        while size > 1024:
            suffixIndex += 1
            size = size/1024.0
        return "%.*f %s" % (precision, size, suffixes[suffixIndex])

    def test(self):
        import tarfile
        arcname = 'backup-1382012_104645.tar.gz'
        ret = os.path.getsize(arcname)
        tar = tarfile.open(arcname, 'r:gz')
        lista = tar.getmembers()
        for n in lista:
            commands.getstatusoutput('rm -rf n')
            tar.extract(n, path="")
        tar.close()
        return GetHumanReadable(ret, 1)

    def new_backup(self):
        #from django.utils.timezone import utc
        import datetime, tarfile

        suffix = '.tar.gz'
        #date = datetime.datetime.utcnow().replace(tzinfo=utc)
        date = datetime.datetime.now()
        file_name = '/daemon-files/backup_minicom-%s%s%s_%s%s%s' % (date.day, date.month, date.year, date.hour, date.minute, date.second)
        # Add the files listed in filelist and close it. 
        try:
            tar = tarfile.open(file_name + suffix, 'w:gz')
            for f in self.filelist:
                tar.add(f, arcname=None, recursive=True, exclude=None)
            tar.close()
        except IOError, e:
            return os.strerror(e.errno)
       
        # Return a dictionary with the name and date of backup file.
        size = os.path.getsize(file_name + suffix)
        human_readable_size = self.GetHumanReadable(size, 1)
        data = {'name': file_name + suffix, 
                'date': date,
                'size': human_readable_size}

#        t = Task()
#        status = t.moveBackupFile(file_name + suffix)
#        if status != 1:
#            return status
        return data

    # def unpack(arcname):
    #     tar = tarfile.open(arcname, 'r:gz')
    #     lista = tar.getmembers()
    #     for n in lista:
    #         commands.getstatusoutput('rm -rf n')
    #         tar.extractall(n, path="")
    #     tar.close()

    # def remove_backup(self, arcname):
    #     # Remove a backup file from disk

    #     output = commands.getstatusoutput('rm %s' % arcname)
    #     if output[0]:
    #         filtered_output = output[1].split(':')
    #         cmd, message, error_msg = filtered_output[0], filtered_output[1], filtered_output[2]
    #         ret = {'command': cmd,
    #                'message': message,
    #                'error_msg': error_msg
    #                }
    #         return ret
    #     return 1
        

