# -*- encoding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required

from django.contrib.auth.models import User
from webadmin.users.models import UserProfile
from webadmin.users.forms import FormUser, FormRegister, SearchForm

from webadmin.tools import Task

def update_context():
    """
    Update context dictionary with
    common informations used to renderize
    HTML templates.
    """
    try:
        """
        List all users, but administrator, 'mc'
        """
        p_all = User.objects.exclude(username__exact='mc')
        total = [u.is_active for u in p_all].count(True)
        context = {'total': total}

        try:
            perfis = [ u.get_profile() for u in p_all ]
            context = {'profiles': perfis,
                       'total': total,
                       }

            if total == 1:
                context.update({'modulo': 'Usuário',
                                'extra': 'ativo'})
            else:
                context.update({'modulo': 'Usuários',
                                'extra': 'ativos'})

        except UserProfile.DoesNotExist:
            pass

    except User.DoesNotExist:
        context = {}

    return context

@login_required
def listing(request):
    if request.method == 'POST':	# Used to search for username, name or email
        form = SearchForm(request.POST)
        if form.is_valid():
            keyword = form.cleaned_data['keyword']

            options = {'username': User.objects.filter(username__exact=keyword),
                 'email': User.objects.filter(email__exact=keyword),
                 'name': User.objects.filter(first_name__exact=keyword),
                 }

            user = []
            for k in options.keys():
                if options[k]:
                    print k,options[k]
                    user = options[k]

            if user:
                context = update_context()
                profiles = [ u.get_profile() for u in user ]
                context.update({'profiles': profiles,
                                'usr': user,
                                'form': form,
                                })
            else:
                context = {'alertmsg': u'Usuário %s não encontrado.' % keyword,
                           'form': form}
                context.update(update_context())
            return render_to_response('user/list.html', context,
                                      context_instance=RequestContext(request))

    form = SearchForm()
    context = {'form': form}
    context.update(update_context())
    return render_to_response('user/list.html', context,
                              context_instance=RequestContext(request))

@login_required
def include(request):
    """
    Add an user in LDAP base and in a profile database.
    """
    # def username_present(username):
    #     if User.objects.filter(username=username).count():
    #         return True
    #     return False

    if request.method == 'POST':
        form_profile = FormUser(request.POST, request.FILES)
        form_reg = FormRegister(request.POST, request.FILES)
        context = {'form_perfil': form_profile,
                   'form_reg': form_reg,
                   }
        if form_profile.is_valid() and form_reg.is_valid():
            username = form_reg.cleaned_data['usuario']
            email = form_reg.cleaned_data['email']
            password = form_reg.cleaned_data['senha']
            quota = form_profile.cleaned_data['quota']
            # IntegrityError: error raised if username exist
 
            # Create a task to be managed by a daemon, 
            # to create a user in LDAP.
            task = Task()
            nok = task.addUser(username, password, quota)
            if nok:
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('user/include.html', context,
                                          context_instance=RequestContext(request))
            else:
                perfil_usuario = form_profile.save(commit=False)

                usuario_sistema = User.objects.create_user(username=username,
                                                           email=email,
                                                           password=password)
                usuario_sistema.is_active = False
                usuario_sistema.is_staff = False
                usuario_sistema.first_name = perfil_usuario.first_name
                usuario_sistema.last_name = perfil_usuario.last_name
                usuario_sistema.save()
                
                perfil_usuario.user_id = User.objects.get(username__exact=username).id
                perfil_usuario.save()
                
                context = {'msg': u'Usuário adicionado com sucesso.'}
                context.update(update_context())
                return render_to_response('user/list.html', context,
                                          context_instance=RequestContext(request))
    else:
        form_profile = FormUser()
        form_reg = FormRegister()
        context = {'form_perfil': form_profile,
                   'form_reg': form_reg,
                   }
    context.update(update_context())
    return render_to_response('user/include.html', context,
                              context_instance=RequestContext(request))

@login_required
def activate(request, nome_usuario):
    try:
        u = User.objects.get(username__exact=nome_usuario)
        u.is_active = True
        u.save()
        mensagem = u'Usuário %s ativado com sucesso.' % u.username
        context = {'msg': mensagem}
        context.update(update_context())
    except User.DoesNotExist:
        pass

    return render_to_response('user/list.html', context,
                              context_instance=RequestContext(request))

@login_required
def block(request, nome_usuario):
    """
    Bloqueia usuário apenas tornando-o inativo para o gerenciador.
    """
    try:
        u = User.objects.get(username__exact=nome_usuario)
        u.is_active = False
        u.save()
        mensagem = u'Usuário %s bloqueado.' % u.username
        context = {'msg': mensagem}
        context.update(update_context())
        return render_to_response('user/list.html', context,
                                  context_instance=RequestContext(request))
    except User.DoesNotExist:
        pass

    return render_to_response('user/list.html', {},
                              context_instance=RequestContext(request))

@login_required
def alter(request, id_usuario):
    usuario = get_object_or_404(UserProfile, pk=id_usuario)
    user = get_object_or_404(User, pk=int(id_usuario)+1)

    form_reg = FormRegister({'usuario': user,
                             'email': user.email,
                             })
    form_profile = FormUser(instance=usuario)
    context = {'form_reg': form_reg,
               'form_perfil': form_profile,
               }
    if request.method == 'POST':
        form_reg = FormRegister(request.POST, {'usuario': user,
                                               'email': user.email,
                                               })
        form_profile = FormUser(request.POST, instance=usuario)
        if form_reg.is_valid() and form_profile.is_valid():
            perfil_usuario = form_profile.save(commit=False)
            username = form_reg.cleaned_data['usuario']
            email = form_reg.cleaned_data['email']
            password = form_reg.cleaned_data['senha']
            quota = form_profile.cleaned_data['quota']

            task = Task()
            ret = task.alterUser(username, password, quota)
            if ret != 1:
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('user/include.html', context,
                                          context_instance=RequestContext(request))

            usuario_sistema = User.objects.get(username__exact=usuario)
            usuario_sistema.username = username
            usuario_sistema.set_password(password)
            usuario_sistema.email = email
            usuario_sistema.first_name = perfil_usuario.first_name
            usuario_sistema.last_name = perfil_usuario.last_name
            usuario_sistema.save()

            perfil_usuario.save()
            form_profile.save_m2m()
            context = {'msg': u'Usuário %s alterado com sucesso.' % username}
            context.update(update_context())
            return render_to_response('user/list.html', context,
                                      context_instance=RequestContext(request))
    else:
        form_reg = FormRegister({'usuario': user,
                                 'email': user.email,
                                 })
        form_profile = FormUser(instance=usuario)
        context = {'form_perfil': form_profile,
                   'form_reg': form_reg,
                   }
    context.update(update_context())
    return render_to_response('user/alter.html', context,
                              context_instance=RequestContext(request))

@login_required
def remove(request, id_usuario):
    """
    Remove a user from system
    """
    context = update_context()
    try:
        u = User.objects.get(id=id_usuario)
        task = Task()
        ret = task.removeUser(u)
        if ret != 1:
            errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
            context.update(update_context())
            context.update({'errmsg': errmsg})
            return render_to_response('user/list.html', context,
                                      context_instance=RequestContext(request))

        u.delete()
        context = {'msg': u'Usuário %s removido com sucesso' % u.username }
        context.update(update_context())
        return render_to_response('user/list.html', context,
                                  context_instance=RequestContext(request))
    except User.DoesNotExist:
        pass
    
    return render_to_response('user/list.html', {},
                              context_instance=RequestContext(request))
